// @ts-ignore
import katex from "https://cdn.jsdelivr.net/npm/katex@0.12.0/dist/katex.mjs";
import {onFirstVisible} from "./Util.js";
/**
 * Custom element `<latex-js>` that renders latex code put inside it using the [Katex library](https://katex.org).
 * 
 * Usable as `<latex-js latex="code"></latex-js>` or via the `.latex` property.
 */
export default class LatexJS extends HTMLElement {
	/**
	 * Constructs a new LatexJS element.
	 * @param {string} latex
	 * @param {{delay?:boolean,macros?:{[key:string]:string}}} options
	 */
	constructor(latex="",{delay=false,macros={}}={}){
		super();
		this.attachShadow({mode:"open"});
		this.shadowRoot.innerHTML = `
			<link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/katex@0.12.0/dist/katex.css" integrity="sha384-qCEsSYDSH0x5I45nNW4oXemORUZnYFtPy/FqB/OjqxabTMW5HVaaH9USK4fN3goV" crossorigin="anonymous">
			<style>
				:host {
					display: block;
				}
				:host > div > .katex {
					font-size: 1em;
				}
			</style>
			<div></div>
		`;
		this._div = this.shadowRoot.querySelector("div");
		this._delay = delay;
		if (delay){
			onFirstVisible(this,()=>{
				this._div.innerHTML = LatexJS.processLatex(this.latex,this._macros);
				this._delay = false;
			});
		}
		this._macros = macros;
		this.latex = latex;
	}

	set latex(latex){
		this.setAttribute("latex",latex)
	}

	get latex(){
		return this.getAttribute("latex");
	}

	set macros(macros){
		this._macros = macros;
		this._div.innerHTML = LatexJS.processLatex(this.latex,this._macros);
	}

	get macros(){
		return this._macros;
	}

	static get observedAttributes(){
		return ["latex"];
	}

	/**
	 * @param {string} name
	 * @param {string} oldValue
	 * @param {string} newValue
	 */
	attributeChangedCallback(name,oldValue,newValue){
		if (name=="latex"){
			if (this._delay){
				this._div.textContent = newValue;
			}else{
				this._div.innerHTML = LatexJS.processLatex(newValue,this._macros);
			}
		}
	}

	/**
	 * Generates a html string from a latex source string.
	 * @param {string} latex
	 * @param {unknown} macros
	 */
	static processLatex(latex,macros={}){
		if (latex){
			let fragments = latex.split("$");
			let html = "";
			fragments.forEach((fragment,index)=>{
				html += index%2?katex.renderToString(fragment,{macros}):fragment;
			});
			return html;
		}else{
			return "";
		}
	}
}
customElements.define("latex-js",LatexJS);